!! cath_prunext.f90 program: combine of distprune.f90 and extend2.f90:
!! 01/28/03 Xin Yuan 
!! 1. Tuning the alignments after nonsequential alignment: by checking the aligned paired residue distance.
!! After removing the bad alignments, then extend the seeds of alignments by checking the distance
!! 2. different from extend.f90: while extend the residues, check the distance first before adding it
!! Input: the overlay outputfile(out.pdb with transformed coordinates) for the best nonsequential alignment (low RMSD, large aligned len) 
!! ./xovrlayali 1fsfA1ig0A.prune2 1fsfA.pdb A 1 1ig0A.pdb A 1 500 out.pdb
!! output: one alignment which has pruned large targ-tmpl distance and been extended at both ends of the aligned block
!! output is used as input for overlay
!!********************************* NOTE ***********************
!! note: targ(1)%resSeq is not necessary to be 1, when storing the alignment into master(), allocate (length of target
!! + targ(1)%resSeq)
!! how to mach the targ%resSeq to seqlist? use the 2nd line from **.gapless, to read the difference between seqlist and resSeq
!! index a for targ(a)% or tmpl(a)% is stored as seqlist number, starts from 1 ends with len1 or len2 
!! ******************************* NOTE**************************
!! alignment in *.ali nonsequential alignment is the resSeq number

module datatype
  type residue
    integer(kind=4) :: resSeq !! use resSeq in the .pdb
    real(kind=4) :: xyzca(3)
    real(kind=4),dimension(3) :: xyzn,xyzc !! coordinates for Ni,CAi,Ci
    real(kind=4) :: phi,psi,omega !!backbone angles for mda checking
    integer(kind=2) :: flag !!flag the residue if it was pruned
  end type residue
  type(residue),allocatable,dimension(:) :: targ  !! note: targ(1)%resSeq is not always 1 it is seqSeq# for seqlist 1 on targ
  type(residue),allocatable,dimension(:) :: tmpl
  !!  i=resSeq on targ  master(i)=resSeq on tmpl   targ(i)% or tmpl(master(i))%xyzca, xyzc,xyzn,phi,psi,omega
  !!  10                   20  ----> blk(1)%startp
  !!  11                   21  ----> blk(1)%endp
  !!  20                   41  ----> blk(2)%startp
  type block_data !!each aligned fragment is a block, used for calculate the block distance sum(d)/n
    integer(kind=4) :: startp,endp !!record the residue position for that block, on x and y axis
    integer(kind=2) :: bflag  !!flag the block if it is pruned in the distanceprune
  end type block_data
  type(block_data)::blk(200)
  integer(kind=4) :: numblk
  integer(kind=4),allocatable,dimension(:) :: master !! nonsequential alignment vector to store the aligned pairs, 
  ! real(kind=4),parameter :: DISTCUT=8.85
  real(kind=4),parameter :: DISTCUT=6.00
  real(kind=4), parameter :: MDACUT=100.
  integer(kind=4),parameter :: MAXRES = 3000
  integer(kind=4) :: len1,len2 !! ******** NOTE ************** len1 and len2 are full length of the structure, 
                               !! not only the domain len diaplayed as the 1st line in gapless, get them from overlay output

CONTAINS
  !!*********
  !! v1,v2 are the vector of xyzca(3) for target and template
  real function getdist(v1,v2)
    implicit none
    real,intent(in) :: v1(3),v2(3)
    real :: vec(3)
    !!write(*,*) v1(1:3), v2(1:3)
    vec = v1 - v2
    getdist = sqrt( sum(vec*vec) )
  end function getdist
  !!**********
  !! mda: max of the angle difference. pos1 and pos2 are aligned extendsion
  !! pos1 is extended residue pos on targ.
  !! pos2 is the extended residue pos on template.
  real function getmda(pos1, pos2, direction)
    implicit none
    integer, intent(in)::pos1, pos2, direction
    real :: mda
    mda=0.
    if(direction == 1) then !!extend upstream, then check the psi(i-1),phi(i),psi(i) for i, not for w(i)
      mda = max(mda, angdiff(targ(pos1-1)%psi, tmpl(pos2-1)%psi))
      mda = max(mda, angdiff(targ(pos1)%phi, tmpl(pos2)%phi))
      mda = max(mda, angdiff(targ(pos1)%psi, tmpl(pos2)%psi))
      getmda=mda
      return
    else  !!extend downstream, then check the phi(i),psi(i),w(i), not psi(i-1)
      mda = max(mda, angdiff(targ(pos1)%phi, tmpl(pos2)%phi))
      mda = max(mda, angdiff(targ(pos1)%psi, tmpl(pos2)%psi))
      mda = max(mda, angdiff(targ(pos1)%omega,tmpl(pos2)%omega))
      getmda=mda
      return
    endif
  end function getmda
  !!***************
  !! get angle difference
  real function angdiff(ang1, ang2)  !!pos11, pos12 is the recnum for s()..
    implicit none
    real(kind=4) :: ang1, ang2 !!angdiff is always 0< < 180
    angdiff=abs(ang1- ang2)
    if(angdiff > 180.) then
      angdiff = 360. - angdiff
    endif
    return
  end function angdiff
  !!*********
  !! targetresSeq  templateresSeq 
  subroutine writeout
    implicit none
    integer :: i
    do i=1,len1
      if (master(i)/=0 .and. targ(i)%flag==0 .and. tmpl(master(i))%flag==0) then
        write(12,'(i5,i5)') targ(i)%resSeq,  tmpl(master(i))%resSeq
      endif
    enddo
  end subroutine writeout

  !! ********
  !! Purpose: get the index for targ% or tmpl% based on resSeq. return the seqlist based on the resSeq, whichs=0 is for target
  integer function findseq(res, whichs)
    implicit none
    integer(kind=4), intent(in) :: res,whichs
    integer(kind=4) :: i
    if(whichs==0) then !! search for the target
      do i=1,len1
        if(targ(i)%resSeq == res) then
          findseq = i
          return
        endif
      enddo
    else
      do i=1,len2
        if(tmpl(i)%resSeq == res) then
          findseq = i
          return
        endif
      enddo
    endif
    !! goes here if nothing was found
    findseq=-1
    return
  end function findseq
  
  !! ********
  !! get the actual full length of len1, len2 from the overlay output (one of input for this program)
  !! read the xyzca(3), xyzc(3),xyzn(3) for each of residues on the superimposed target and template
  !! input file: is the .pdb ooutput from overlay program
  subroutine readcoord(imposefile)
    implicit none
    character(len=80), intent(in) :: imposefile
    character(len=80) :: aline
    integer(kind=4) :: i, j, ios,res,seq

    !! open the superimposed overlay.pdb file to read the coordinates for each residue
    open(9, file = imposefile, status = "old", iostat = ios) 
    if(ios/=0) stop "Error openning overlay.pdb input file"
    !!do
      !! first: read the coordinates for template 
      !!if(aline(1:3)/="TER") then
    !! ********************************* NOTE ***************************
    !! when read in the coordinates , should match the resSeq for each position, not seglist
    len1=0
    len2=0
      do
        read(9,'(a)',iostat=ios) aline
        if(ios/=0) exit
        if(aline(1:3)=="TER") exit !!goes here if the templates ends
        !! read for template first: get len2,tmpl(len2)%resSeq, read N, CA, C 
        if(aline(14:15)=="N ") then
          len2=len2+1
          read(aline(23:26),'(i4)') tmpl(len2)%resSeq
          seq=len2
          !!seq = findseq(res,1)
          !!if(seq<0) stop "Error in finding the resSeq in template"
          read(aline(31:54),'(3f8.3)') tmpl(seq)%xyzn(1:3)
          !!write(*,*) seq,tmpl(seq)%resSeq, tmpl(seq)%xyzn(1:3)
        elseif(aline(14:15)=="CA") then
          !!read(aline(23:26),'(i4)') res
          read(aline(31:54),'(3f8.3)') tmpl(seq)%xyzca(1:3)
          !!write(*,*) seq,tmpl(seq)%resSeq, tmpl(seq)%xyzca(1:3)
        elseif(aline(14:15)=="C ") then
          !!read(aline(23:26),'(i4)') res
          read(aline(31:54),'(3f8.3)') tmpl(seq)%xyzc(1:3)
          !!write(*,*) seq,tmpl(seq)%resSeq, tmpl(seq)%xyzc(1:3)
        endif
        !!if(seq > len2+1) stop "Error in reading the coordinates, exceeds len2 of template"
        if(len2 > MAXRES+1) stop "Error in template, len2 exceeds MAXRES"
      enddo
      !!else !! goes here if meets "TER"--now read for target: get the len1, targ(len1)%resSeq, the coordinates for the target
      !!write(*,*) " Template ends: len2=",len2
      !!write(*,*) " -----------------------------------------"
      read(9,'(a)',iostat=ios) aline
      if(ios/=0) stop "Error in overlay.pdb"
      do
        if(aline(14:15)=="N ") then
          len1=len1+1
          read(aline(23:26),'(i4)') targ(len1)%resSeq
          seq=len1
          !!seq = findseq(res,0)
          !!if(seq<0) stop "Error in finding the resSeq in target"
          read(aline(31:54),'(3f8.3)') targ(seq)%xyzn(1:3)
          !!write(*,*) seq,targ(seq)%resSeq, targ(seq)%xyzn(1:3)
        elseif(aline(14:15)=="CA") then
          !!read(aline(23:26),'i') res
          read(aline(31:54),'(3f8.3)') targ(seq)%xyzca(1:3)
          !!write(*,*) seq,targ(seq)%resSeq, targ(seq)%xyzca(1:3)
        elseif(aline(14:15)=="C ") then
          !!read(aline(23:26),'i') res
          read(aline(31:54),'(3f8.3)') targ(seq)%xyzc(1:3)
          !!write(*,*) seq,targ(seq)%resSeq, targ(seq)%xyzc(1:3)
        endif
        !!if(seq > len1+1) stop "Error in reading the coordinates, exceeds len of target"
        if(len1 > MAXRES+1) stop "Error in target, len1 exceeds MAXRES"
        read(9,'(a)',iostat=ios) aline
        if(ios/=0) exit
        if(aline(1:3)=="TER") exit
      enddo
      !!write(*,*) " Target ends: len1=",len1
    !! for debugging
    !!do i=1, len1
    !!  write(*,*) " ",i, targ(i)%resSeq,targ(i)%xyzn(1:3)
    !!  write(*,*) " ",i, targ(i)%resSeq,targ(i)%xyzca(1:3)
    !!  write(*,*) " ",i, targ(i)%resSeq,targ(i)%xyzc(1:3)
    !!enddo
    !!write(*,*) "-------------------------------------"
    !!do i=1, len2
    !!  write(*,*) i, tmpl(i)%resSeq,tmpl(i)%xyzn(1:3)
    !!  write(*,*) i, tmpl(i)%resSeq,tmpl(i)%xyzca(1:3)
    !!  write(*,*) i, tmpl(i)%resSeq,tmpl(i)%xyzc(1:3)
    !!enddo
    write(*,*) "--Finish reading in the coordinates"
  end subroutine readcoord

  !! *****
  !! alignment is stored in master(): i-->master(i), where i is the seqlist for target, and master(i) is seqlist for template
  subroutine readalignment(nonseqfile)
    implicit none
    character(len=80),intent(in) :: nonseqfile
    integer(kind=4) :: num, ios,i,j,targres,tmplres
    character(len=80) :: aline
    !! read the single nonsequential alignments
    open(10, file = nonseqfile, status = "old", iostat = ios) 
    if(ios/=0) stop "Error openning the alignment file"
    !!****************** NOTE ********************
    !! index a for master(a) is the seqlist of the target, 
    !! where master(a) is the seqlist for template
    allocate(master(MAXRES), stat=ios)
    if(ios/=0) stop "Error in allocating master()"
    !!write(*,*) "In readalignment: len1=",len1," len2=",len2
    num=0
    master=0
    !! ************* NOTE *****************
    !! alignment in nonseq alignment file is the resSeq number
    do
      read(10, '(a)', iostat = ios) aline
      if(ios /= 0) exit
      num=num+1
      read(aline,*,iostat=ios) targres,tmplres  !! resSeq for target: switch to seqlist, call findseq 
      if (ios/=0) stop 'cathpruneext.f90:: error reading alignment file.'
      i = findseq(targres, 0)
      if( i <0) stop "cathpruneext.f90::Error in finding targres"
      !! read(aline(6:10), '(i5)') tmplres  !! resSeq for template: switch to seqlist, call findseq 
      master(i) = findseq(tmplres, 1)  !! seqlist for template: to access resSeq, use targ(master(i))%resSeq
      if( master(i) <0) stop "cathpruneext.f90::Error in finding tmplres"
      !!write(*,*) i,"-->",master(i)
    enddo
    !!write(*,*) "num=",num
    !! for debug
    !!write(*,*) "the following is the original alignment"
    !!do i=1,len1
    !!  if(master(i)==0) then
    !!    write(*,*) i, targ(i)%resSeq, master(i),0
    !!  else
    !!    write(*,*) i, targ(i)%resSeq, master(i),tmpl(master(i))%resSeq
    !!  endif
    !!enddo
    !! define the alignment blocks based on the i--master(i) aligned fragments
    call markblk
    !!write(*,*) "after marking the blocks, numblk=",numblk
  end subroutine readalignment
  !! *****
  subroutine distanceprune
    implicit none
    integer(kind=4) :: i,j
    real(kind=4) :: avgdist
    !! check the average distance for that block
    do i=1, numblk !! i is the index for blk, which is also the block label on x-axis
      do j=blk(i)%startp,blk(i)%endp !! j is the resSeq for target, master(j) is the resseq for template
        avgdist=avgdist+getdist(targ(j)%xyzca, tmpl(master(j))%xyzca)
      enddo
      avgdist=avgdist/(blk(i)%endp-blk(i)%startp+1)
      write(*,'("at block",i3,i5,"-",i5," avgdist=",f8.3)') i,targ(blk(i)%startp)%resSeq, targ(blk(i)%endp)%resSeq, avgdist
      if(avgdist > DISTCUT) then !! if this alignment is bad, then flag all the residues on the block
        blk(i)%bflag=1
        do j=blk(i)%startp,blk(i)%endp !! j is the index for align(), align(j)%trg is the res pos in target
          targ(j)%flag=1
          tmpl(master(j))%flag=1
          write(*,*) " *** flag", " block", i, targ(j)%resSeq,"-->",tmpl(master(j))%resSeq
        enddo
      endif
    enddo
    write(*,*) "--Finish distance pruning"
    !! write out the pruned alignment to the *.prune output file
    do i=1,len1
      if (master(i)/=0 .and. targ(i)%flag==1 .or. tmpl(master(i))%flag==1) then
        write(11,'(i5,i5)') targ(i)%resSeq,  tmpl(master(i))%resSeq
      endif
    enddo
  end subroutine distanceprune
  !! ***
  subroutine angleprune
    implicit none
    integer(kind=4) :: i,j
    real(kind=4) :: avgmda
    !! check the average distance for that block
    do i=1, numblk !! i is the index for blk, which is also the block label on x-axis
      if(blk(i)%bflag==1) cycle
      do j=blk(i)%startp,blk(i)%endp !! j is the resSeq for target, master(j) is the resseq for template
        avgmda=avgmda+getmda(j, master(j),1)
      enddo
      avgmda=avgmda/(blk(i)%endp-blk(i)%startp+1)
      write(*,*) "at block",i," avgmda=",avgmda
      if(avgmda > MDACUT) then !! if this alignment is bad, then flag all the residues on the block
        !!write(*,*) "at block",i," avgmda=",avgmda
        blk(i)%bflag=1
        do j=blk(i)%startp,blk(i)%endp !! j is the index for align(), align(j)%trg is the res pos in target
          targ(j)%flag=1
          tmpl(master(j))%flag=1
          write(*,*) " $$$ aflag", " block", i, targ(j)%resSeq,"-->",tmpl(master(j))%resSeq
        enddo
      endif
    enddo
    write(*,*) "--Finish mda pruning"
    !! write out the pruned alignment to the *.prune output file
    !!write(11,*) "----------------------- mda pruned ------------------"
    do i=1,len1
      if (master(i)/=0 .and. targ(i)%flag==1 .or. tmpl(master(i))%flag==1) then
        write(11,'(i5,i5)') targ(i)%resSeq,  tmpl(master(i))%resSeq
      endif
    enddo
  end subroutine angleprune


  !!*************
  !! after storing the full alignments into the align(), get the blk data first, then
  !! calculate the average distance for each blk, if > DISTCUT, reject the whole blk
  !! i--> master(i)
  !! 1     10 -->blk(1)
  !! 2     11
  !! 3      0
  !! 4     29 -->blk(2)
  !! 5     30
  !  6     40 -->blk(3)
  subroutine markblk
    implicit none
    integer :: i,b,m,prevtrg,prevtml
    blk(:)%startp=0
    blk(:)%endp=0
    blk(:)%bflag=0
    b=1
    numblk = 0
    !! go over every aligned fragment on the target to get the blocks, label them,
    !! and get the start and end position for each block
    !! keep the temporary start resSeq for the first alignmnet
    do i=1,len1
      if(master(i)/=0) then
        prevtrg = i
        blk(b)%startp=prevtrg
        prevtml = master(i)  !! 1-->10, i=1, master(1) = 10
        exit
      endif
    enddo
    
    do i=prevtrg+1, len1 !! go over through the whole alignment 
      if(master(i)==0) cycle !! if it is not aligned, go to the next residue 
      !! if the prev and current residue on either trg or template is not consecutive
      if(i-prevtrg/=1 .or. master(i)-prevtml/=1) then
        blk(b)%endp = prevtrg !! %endp is resSeq for target
        b=b+1
        blk(b)%startp = i !! %start is resSeq for target 
      endif
      prevtrg = i !! keep the prev position on target
      prevtml = master(i) !! keep the prev position on template
    enddo
    blk(b)%endp = prevtrg
    numblk = b
    write(*,*) "b=",b
    !! go over the blocks on y axis, and get the block label on y axis for each block
    !!do i=1, b !! i is the index for blk, which is also the block label on x-axis
    !!  write(*,*) "blk=",i,"start=",blk(i)%startp,"end=",blk(i)%endp
    !!enddo
  end subroutine markblk
  
  !! ****
  !! extend each alignment block. While adding the paired residues, check the distance
  subroutine extend
    implicit none
    integer(kind=4) :: i, j, prev,gap,newtmpl
    real :: dist,mda
    dist=0.
    mda=0.
    !! go over each block, if there is a gap between the end of prev block and the start of the current blk, then extend
    !! by checking the distance and mda of the newly paired targ-tmpl 
    do i=1,numblk
      !! extend the gaps
      if(blk(i)%bflag==1) cycle
      if( i/=numblk .and. blk(i+1)%startp-blk(i)%endp > 1) then
        !! extends toward upstream from the end of current blk
        prev = blk(i)%endp
        do gap=1,blk(i+1)%startp-blk(i)%endp-1 !! add towards upstream
          if(prev+gap > len1 .or. master(prev)+gap > len2 .or. master(prev)+gap==master(blk(i+1)%startp)) exit
          dist=getdist(targ(prev+gap)%xyzca,tmpl(master(prev)+gap)%xyzca)
          write(*,*) "try extend up: ", targ(prev)%resSeq+gap,"->",tmpl(master(prev))%resSeq+gap, "dist=",dist
          if(prev+gap/=len1 .and. master(prev)+gap /= len2) then
            mda = getmda(prev+gap, master(prev)+gap,1)
          endif
          !!write(*,'("  xyz ", 6f8.3)') targ(prev+gap)%xyzca(1:3),tmpl(master(prev)+gap)%xyzca(1:3)
          !!write(*,'("  ang ", 6f10.3)') targ(prev+gap)%phi,targ(prev+gap)%psi,targ(prev+gap)%omega, &
          !!   tmpl(master(prev)+gap)%phi,tmpl(master(prev)+gap)%psi,tmpl(master(prev)+gap)%omega
          !!write(*,*) prev+gap,master(prev)+gap, "dist=",dist, "  mda=",mda

          if(dist < DISTCUT .and. mda < MDACUT) then
            master(prev+gap) = master(prev)+gap 
            write(*,'(" + ",i5,i5)') prev+gap,master(prev)+gap
          else !! ** no jump extention: don't extend once it is very distant
            exit
            write(*,*)
          endif
        enddo
        !! extends towards downstream from start of current block 
        if(i==1 .and. blk(1)%startp==1) cycle
        if(i==1 .and. blk(1)%startp > 1) then
          prev = blk(1)%startp
          gap = prev-1
        else
          prev = blk(i)%startp
          gap=blk(i)%startp-blk(i-1)%endp-1 !! add towards downstream
        endif
        do j=1,gap !! add towards downstream, j is the steps
          newtmpl= master(prev)-j
          if(newtmpl < 1 .or. newtmpl==blk(i-1)%endp) exit
          !! prev-j is the position on target, master(prev)-j is the pos on template
          write(*,*) "try extend down: ",targ(prev)%resSeq-j, "->",tmpl(master(prev))%resSeq-j
          dist=getdist(targ(prev-j)%xyzca,tmpl(newtmpl)%xyzca)
          write(*,*) " dist=",dist
          if(prev-j/=1 .and. master(prev)-j /= 1) then
            mda = getmda(prev-j, master(prev)-j,0)
          endif
          !!write(*,'("xyz ", 6f8.3)') targ(prev-j)%xyzca(1:3),tmpl(master(prev)-j)%xyzca(1:3)
          !!write(*,'("ang ", 6f10.3)') targ(prev-j)%phi,targ(prev-j)%psi,targ(prev-j)%omega, &
          !!   tmpl(master(prev)-j)%phi,tmpl(master(prev)-j)%psi,tmpl(master(prev)-j)%omega
          write(*,*) prev-j,master(prev)-j, "dist=",dist, "  mda=",mda
          if(dist < DISTCUT .and. mda < MDACUT) then
            master(prev-j) = master(prev)-j
            write(*,'(" +++ ",i5,i5)') prev-j,master(prev)-j
          else !! ** no jump extention: don't extend once it is very distant
            exit
            write(*,*)
          endif
          !!write(10,'(i5,i5)') prev+gap,master(prev)+gap
          !!write(10,'(i5,i5)') prev+gap,master(prev)+gap
        enddo
      endif
    enddo 
    !! extend up from the end of the last alignment
    if(len1-blk(numblk)%endp > 1) then
      prev = blk(numblk)%endp
      do gap=1,len1-prev-1 !! add towards upstream
        if(prev+gap > len1 .or. master(prev)+gap > len2 .or. master(prev)+gap==master(len1)) exit
        dist=getdist(targ(prev+gap)%xyzca,tmpl(master(prev)+gap)%xyzca)
        write(*,*) "try extend last block up: ", targ(prev)%resSeq+gap,"->",tmpl(master(prev))%resSeq+gap, "dist=",dist
        if(prev+gap/=len1 .and. master(prev)+gap /= len2) then
          mda = getmda(prev+gap, master(prev)+gap,1)
        endif
        !!write(*,'("  xyz ", 6f8.3)') targ(prev+gap)%xyzca(1:3),tmpl(master(prev)+gap)%xyzca(1:3)
        !!write(*,'("  ang ", 6f10.3)') targ(prev+gap)%phi,targ(prev+gap)%psi,targ(prev+gap)%omega, &
        !!   tmpl(master(prev)+gap)%phi,tmpl(master(prev)+gap)%psi,tmpl(master(prev)+gap)%omega
        !!write(*,*) prev+gap,master(prev)+gap, "dist=",dist, "  mda=",mda

        if(dist < DISTCUT .and. mda < MDACUT ) then
          master(prev+gap) = master(prev)+gap
          write(*,'(" + ",i5,i5)') prev+gap,master(prev)+gap
        else !! ** no jump extention: don't extend once it is very distant
          exit
        endif
      enddo
    endif
    call writeout
  end subroutine extend
end module datatype

program cathprunext
  use datatype
  use vectormath
  implicit none
  character(len=200) :: aline, bline
  integer(kind=4) :: jarg, iargc
  integer(kind=4) :: i, j, ios, gap,m,n,num,prev,res,tprev
  character(len=80) :: alignfile,ovlyfile,output1,output2,gaplessfile
  ! real(kind=4) :: dist
  !! real(kind=4) :: measure !! vectormath
  character(len=6) :: targ_name,tmpl_name
  integer(kind=4) :: targ_seqlist, targ_resSeq,tmpl_seqlist,tmpl_resSeq, diff

  jarg = iargc()
  if(jarg < 2) then
    write(*,*) './xcath_prunexd singlenoseqalign ovrlyout.pdb [.extend output  .prune output]' 
    stop 'cath_prunexd.f90 Jan-03'
  endif
  call getarg(1, alignfile)
  call getarg(2, ovlyfile)
  if(jarg > 2) then
    call getarg(3, output2)
    call getarg(4, output1)
  else
    write(output1,'("junk.prune")') 
    write(output2,'("junk.extend")') 
  endif
  write(*,'("Input alignment file=",a30)') alignfile
  write(*,'("Input pdb file after overlay=",a30)') ovlyfile
  write(*,'("Output pruned alignment file=",a30)') output1
  write(*,'("Output extended alignment file=",a30)') output2
  open(11, file = output1, status = "replace", iostat = ios) 
  if(ios/=0) stop "Error openning the .prune file"
  open(12, file = output2, status = "replace", iostat = ios) 
  if(ios/=0) stop "Error openning the .extend file"
  !! static allocate for targ()% and tmpl()%
  allocate(targ(MAXRES), stat=ios)
  if(ios/=0) stop "Error in allocating targ()"
  allocate(tmpl(MAXRES), stat=ios)
  if(ios/=0) stop "Error in allocating tmpl()"
  targ%resSeq=0
  tmpl%resSeq=0

  !!*************************************************************************************************
  !!*** WRONG:  get the length of the target and the template from the first line of **.gapless file*
  !!open(8, file = gaplessfile, status = "old", iostat = ios) 					    *
  !!if(ios/=0) stop "Error openning the alignment file"                                             *
  !!read(8,'(a)',iostat=ios) aline								    *
  !!if(ios/=0) stop "Error in gapless file"							    *
  !!read(aline(1:7), 'i') len1									    *
  !!read(aline(8:14), 'i') len2									    *
  !!write(*,*) "len1=",len1,"len2=",len2							    *
  !!allocate(targ(len1), stat=ios)
  !!if(ios/=0) stop "Error in allocating targ()"
  !!allocate(tmpl(len2), stat=ios)
  !!if(ios/=0) stop "Error in allocating tmpl()"
  !!targ%resSeq=0
  !!tmpl%resSeq=0

  !! get the difference between resSeq and seqlist for the target and the template from the 2nd line of **.gapless file 
  !! then initialize all the resSeq
  !!read(8,'(a)',iostat=ios) aline
  !!if(ios/=0) stop "Error in gapless file"
  !!read(aline(14:19), 'a5') targ_name
  !!read(aline(22:25), 'i') targ_seqlist
  !!read(aline(28:31), 'i') targ_resSeq
  !!read(aline(33:38), 'a5') tmpl_name
  !!read(aline(41:44), 'i') tmpl_seqlist
  !!read(aline(47:50), 'i') tmpl_resSeq
  !!write(*,*) targ_name, targ_seqlist, targ_resSeq, tmpl_name,tmpl_seqlist,tmpl_resSeq
  !!diff = targ_resSeq - targ_seqlist
  !!do i=1, len1
  !!  targ(i)%resSeq = i+diff
  !!  write(*,*) " * ", i,targ(i)%resSeq  
  !!enddo
  !!diff = tmpl_resSeq - tmpl_seqlist
  !!do i=1, len2
  !!  tmpl(i)%resSeq = i+diff
  !!  write(*,*) " * ", i,tmpl(i)%resSeq  
  !!enddo
  !!*************************************************************************************************

  !! read the coordinates for each residues in aligned target and template after superimposed with overlay program
  !! actual len1 and len2 are defined in readcoord(), assign targ%resSeq and tmpl%resSeq
  call readcoord(ovlyfile)

  write(*,*) " In main: after readcoord(): len1=",len1," len2=",len2

  !! calculate the backbone angles for each residues after reading in the coordinates
  do i=1, len1 !! for target
    if(i/=1) then
      targ(i)%omega = measure(targ(i-1)%xyzca,targ(i-1)%xyzc,targ(i)%xyzn,targ(i)%xyzca)
      targ(i)%phi = measure(targ(i-1)%xyzc,targ(i)%xyzn,targ(i)%xyzca,targ(i)%xyzc)
    endif
    !!write(*,*) "targ phi", i, targ(i)%phi
    if(i/=len1-1) then
      targ(i)%psi = measure(targ(i)%xyzn,targ(i)%xyzca,targ(i)%xyzc,targ(i+1)%xyzn)
    endif
  enddo
  do i=1, len2 !! for target
    if(i/=1) then
      tmpl(i)%phi = measure(tmpl(i-1)%xyzc,tmpl(i)%xyzn,tmpl(i)%xyzca,tmpl(i)%xyzc)
    endif
    !!write(*,*) "tmpl phi", i, tmpl(i)%phi
    if(i/=len1-1) then
      tmpl(i)%psi = measure(tmpl(i)%xyzn,tmpl(i)%xyzca,tmpl(i)%xyzc,tmpl(i+1)%xyzn)
      tmpl(i)%omega = measure(tmpl(i)%xyzca,tmpl(i)%xyzc,tmpl(i+1)%xyzn,tmpl(i+1)%xyzca)
    endif
  enddo
  
  !! asign the alignment into the master()
  call readalignment(alignfile)
  !! prune the bad alignments by calculating the average distance difference of the paired residues-each targ/tmpl
  call distanceprune
  !! prune the bad alignments by calculating the average angle difference of the paired residues-each targ/tmpl
  call angleprune

  !! extend each alignment block. While adding the paired residues, check the distance
  call extend
  !!write(*,*) " final prev=",prev
  !!do i=1,len1
  !!  if(master(i)==0) cycle
  !!  !!write(12,'(i5,i5)') i, targ(i)%resSeq, master(i), tmpl(master(i))%resSeq
  !!  write(12,'(i5,i5)') i, master(i)
  !!enddo

  !!close(8)
  close(9)
  close(10)
  close(11)
  close(12)
  deallocate(master)
  deallocate(targ)
  deallocate(tmpl)

end program cathprunext
