!! Written by Namdi Macfoy
!! 2007
MODULE data_organize

    IMPLICIT NONE

!!! declare global data TYPES

    real,parameter :: mindis=8.0, cutoff =4.5

    TYPE residues
        CHARACTER (3) :: aa_abrev
        REAL, ALLOCATABLE,  DIMENSION (:,:) :: atompos
    END TYPE residues
    
    TYPE numatoms
        CHARACTER (3) :: aa_abrev
        INTEGER :: numatoms
    END TYPE numatoms
 
    TYPE point
        REAL :: x, y, z
    END TYPE point

!!! declare global variables

    INTEGER :: i, j, ios, resnum, anum
    INTEGER :: MAXres, FIRSTres
    
    CHARACTER :: chainid

    TYPE (residues), ALLOCATABLE, DIMENSION (:):: residue
    
    CHARACTER (3), PARAMETER, DIMENSION (20) :: abrev =(/'ALA','CYS','ASP','GLU','PHE','GLY','HIS','ILE','LYS', &
        & 'LEU','MET', 'ASN','PRO','GLN','ARG','SER','THR','VAL','TRP','TYR'/)

    INTEGER :: amino(20)= (/5,6,8,9,11,4,10,8,9,8,8,8,7,9,11,6,7,7,14,12/)

    INTEGER, ALLOCATABLE, DIMENSION (:, :) :: SCcontact_map 
    INTEGER, ALLOCATABLE, DIMENSION (:, :) :: CAcontact_map 

END MODULE data_organize

PROGRAM pdb_extraction

    USE data_organize
    IMPLICIT NONE

    
    CHARACTER (100) :: pdbfilename
    CHARACTER (100) :: gcbinfile, gcboutfile

    integer :: jarg, iargc

    jarg = iargc()
    if (jarg < 4) then
       write(*,*) "Usage: xconvertcij PDBfile chainID GCAin GCAout"
       stop 'convertgca,f90 v.1 Sun Sep 11 07:35:21 EDT 2005'
    endif

!!!!!!Enter File Name!!!!!!!!!!!!

    CALL GETARG(1, pdbfilename)    
    OPEN(UNIT=3, FILE=pdbfilename, STATUS="OLD", IOSTAT=ios)
    if (ios/=0) stop 'convertgca:: error opening PDB file.'
    CALL GETARG (2, chainid)
    if (chainid=="_") chainid = " "
    
    CALL parse()
   

    CALL contacts()
    
!    CALL print_contacts

!    CALL print_requests
    
    CALL GETARG (3, gcbinfile)    
    CALL GETARG (4, gcboutfile)
    OPEN(UNIT=4, FILE=gcbinfile, STATUS="OLD", IOSTAT=ios)
    if (ios/=0) stop 'convertgca:: error opening GCAinfile'
    OPEN(UNIT=7, FILE=gcboutfile, STATUS="REPLACE", ACTION="READWRITE",IOSTAT =ios)
    if (ios/=0) stop 'convertgca:: error opening GCAoutfile'
    

    CALL contacts_to_gcb()

    WRITE (*,*) "Contacts successfully embedded into ", gcboutfile
    if (allocated(SCcontact_map)) DEALLOCATE (SCcontact_map)
    if (allocated(CAcontact_map)) DEALLOCATE (CAcontact_map)
    if (allocated(residue)) DEALLOCATE (residue)

END PROGRAM pdb_extraction


!!!!!!! PROCEDURES

SUBROUTINE print_requests()
    
    USE data_organize

    CHARACTER :: query

!!!!!!Print Requests!!!!!!!!!!!!!
    
    DO
    
        WRITE (*,*) " "
        WRITE (*,'( "Please type in request:  ",$)')
        READ (*,'(I4, I2)') resnum, anum

        WRITE (*,*) " "
        
        WRITE (*,'(A15, 2X, A10, 2X, A4, 4X, A1, 9X, A1, 9X, A1)') "RESIDUE NUMBER","AMINO ACID","ATOM","X","Y","Z"
    
        WRITE (*,'(A15, 2X, A10, 2X, A4,4X, A26)') "--------------","----------","----", &
                               & "______________________________"

        WRITE (*,'(I5, 12X, A3, $)') resnum, residue(resnum)%aa_abrev
        
        IF (anum==0) THEN
            
            WRITE (*,'(8X, I2, 5X, 3(F8.3,2X))') 1, residue(resnum)%atompos(1,1:3)         
            DO j=2, SIZE (residue(resnum)%atompos, 1)
                WRITE (*,'(28X, I2, 5X, 3(F8.3,2X))') j, residue(resnum)%atompos(j,1:3)
            END DO
        
        ELSE
            WRITE (*,'(9X, I1, 5X, 3(F8.3,2X))') anum, residue(resnum)%atompos(anum,1:3)         
        END IF
        
        WRITE (*,*) " "
        WRITE (*,'("Would you like to do another query? (y/n)", $)')
        READ (*,*) query
        IF (query == 'n') EXIT

    END DO

END SUBROUTINE print_requests
    

!!!!!Parse within File for xyz coordinates!!!!!!!!!!

SUBROUTINE parse()

    USE data_organize
    IMPLICIT NONE
    
    CHARACTER (80) :: aline
    CHARACTER (3) :: aa_abrev
    INTEGER ::  res_num=0, res_num1, d, c=0
    REAL :: x, y, z
    LOGICAL:: firstentry=.true.        
    j=1

!!!Search through file and find the FIRST residue and the MAX
        
    DO
        READ (3,'(A)', IOSTAT=ios) aline
        IF (ios/=0)EXIT
        IF (aline(1:4) == "ATOM") THEN
            IF (aline(22:22) /= chainid) cycle
            IF (aline(14:14) == "H") cycle  !! ignore hydrogens
            IF (firstentry) READ (aline(23:26), '(I4)') FIRSTres
            firstentry=.false.
            READ (aline(23:26), '(I4)') MAXres
        END IF
    END DO
    
    firstentry = .true.
    !! diagnostic
    ! write(*,*) "Min, Max=",FIRSTres,MAXres

    ALLOCATE (residue(FIRSTres:MAXres), stat = ios); if (ios/=0) stop 'convertgca: Error allocating residue'

    REWIND 3
    
    res_num=-9999
    DO
        READ (3,'(A)', IOSTAT=ios) aline
        IF (ios/=0) EXIT
        !!diagnostic
        ! write(0,'(a)') trim(aline)
        res_num1=res_num
        IF (aline(1:4) == "ATOM") THEN
            IF (aline(22:22) /= chainid) cycle
            IF (aline(14:14) == "H") cycle  !! ignore hydrogens
            READ (aline(18:55), '(A3, 2X, I4, 4X, 3F8.3)', iostat = ios) aa_abrev, res_num, x, y, z
            firstentry=.false.            
            DO d=1, 20
                IF (abrev(d)==aa_abrev) EXIT
            END DO
            if (d>20) stop 'BUG in convertgca'
!!!!!!IF IT IS A NEW AMINO ACID
            IF (res_num/=res_num1) THEN    
                if (allocated(residue(res_num)%atompos)) deallocate(residue(res_num)%atompos)
                ALLOCATE (residue(res_num)%atompos(amino(d),3), stat = ios)
                j=1
                residue(res_num)%aa_abrev=aa_abrev
                residue(res_num)%atompos(:,:)=0.
                residue(res_num)%atompos(j,1:3)=(/x, y, z/)    
!!!!!!OTHERWISE ADD TO ATOMS
!! NOTE: this assumes no multiple occupancies! 
            ELSE
                if (.not.allocated(residue(res_num)%atompos)) stop 'BUG allocating atompos.'
                IF (j<amino(d)) THEN
                    j=j+1
                    residue(res_num)%atompos(j,1:3)=(/x, y, z/)        
                else
                  write(*,*) j-amino(d)," Extra atoms in residue ",res_num
                END IF
            END IF
        END IF
        IF (.not.firstentry .and. aline(1:4)=="TER ") EXIT
    END DO
END SUBROUTINE parse
    
    
SUBROUTINE contacts()
    
    USE data_organize
    IMPLICIT NONE

    INTEGER ::  iatom, jatom, ires, jres, p=0
    REAL ::distance, x, x1, y, y1, z, z1
    
    write(0,*) "================= Firstres, Maxres=", FIRSTres, MAXres
    if (allocated(SCcontact_map)) deallocate(SCcontact_map)
    ALLOCATE (SCcontact_map(FIRSTres:MAXres,FIRSTres:MAXres))
    if (allocated(CAcontact_map)) deallocate(CAcontact_map)
    ALLOCATE (CAcontact_map(FIRSTres:MAXres,FIRSTres:MAXres))

!!! Initialize contact maps to zero

    SCcontact_map(FIRSTres:MAXres,FIRSTres:MAXres)=0
    CAcontact_map(FIRSTres:MAXres,FIRSTres:MAXres)=0

!!! Loop through each residue and their associated atoms, not including the next three residues as contacts
        
    !write(0,*) "MAXres=",MAXres
    DO ires=FIRSTres, MAXres-3
      if (.not.allocated(residue(ires)%atompos)) cycle  !! ignore missing residues
      if (SIZE(residue(ires)%atompos,1) < 4) cycle  !! ignore residues with too many missing atoms
      DO jres=ires+3, MAXres
                          !! diagnostic
            !               WRITE (0,*) "SIZE j",jres,SIZE(residue(jres)%atompos, 1)
            if (.not.allocated(residue(jres)%atompos)) cycle  !! ignore missing residues
            if (SIZE(residue(jres)%atompos,1) < 4) cycle  !! ignore residues with too many missing atoms
            DO jatom=1, SIZE(residue(jres)%atompos, 1)
              if (all(residue(jres)%atompos(jatom,1:3)==0.)) cycle  !! ignore missing atoms

              !!! Test all atoms except N, C and O
              IF (jatom==2 .OR. jatom>4) THEN
                x1=residue(jres)%atompos(jatom, 1)
                y1=residue(jres)%atompos(jatom, 2)
                z1=residue(jres)%atompos(jatom, 3)
                DO iatom=1, SIZE(residue(ires)%atompos, 1)
                    if (all(residue(ires)%atompos(iatom,1:3)==0.)) cycle
                    IF (iatom==2 .OR. iatom>4) THEN
                        x=residue(ires)%atompos(iatom, 1)-x1
                        y=residue(ires)%atompos(iatom, 2)-y1
                        z=residue(ires)%atompos(iatom, 3)-z1
                        distance = SQRT( x*x + y*y + z*z )
                          !! diagnostic
                           ! WRITE (*,*) "DISTANCE",ires, jres, distance
                        IF (distance <= cutoff) THEN
                          SCcontact_map(ires, jres) =1 
                          SCcontact_map(jres, ires) =1 
                        END IF                        
                        IF (distance<=mindis.and.jatom==2.and.iatom==2) then
                          CAcontact_map(ires, jres)=1
                          CAcontact_map(jres, ires)=1
                        END IF                        
                    END IF
                END DO                            
            END IF
          END DO
      END DO    
    END DO
END SUBROUTINE contacts
    
SUBROUTINE print_contacts()
    
    USE data_organize
    IMPLICIT NONE

    DO i=FIRSTres, MAXres
        WRITE (*,'(I2, $)') i
        WRITE(*,'(A1, $)') " "
        
        DO j=FIRSTres, MAXres
            WRITE (*,'(I1, $)') SCcontact_map(i,j)
            IF (mod(j,10)==0)WRITE (*,'(A3, $)') " | "
        END DO
        
        WRITE (*,*) " "
    END DO

END SUBROUTINE print_contacts

SUBROUTINE contacts_to_gcb()

    USE data_organize
    IMPLICIT NONE

    INTEGER::  numcontacts, gca_label, pdb_label, p=0, h, start, gcaMAX, gcaFIRST, pdbFIRST
    CHARACTER (3000):: aline
    CHARACTER (200) :: outputstr
    CHARACTER :: resid
    LOGICAL :: firstinstance =.true.
    REAL :: x,y,z,phi,psi,omg

    TYPE label
        INTEGER :: gca_label
        INTEGER :: pdb_label
    END TYPE label

    TYPE(label), ALLOCATABLE, DIMENSION (:) :: labels

    DO        !! assume gca file starts with 1
       READ (4,'(A)', IOSTAT=ios) aline
       IF (ios/=0) EXIT
       IF (aline(1:7) == "RESIDUE") THEN
    !        IF (firstinstance) READ (aline(10:),*) gcaFIRST
    !        firstinstance=.false.
           READ(aline(10:),*) gcaMAX
       END IF
    END DO

    !firstinstance=.true.

    gcaFIRST = 1
    !gcaMAX=gcaMAX-gcaFIRST+1
    ALLOCATE (labels(gcaMAX),stat=ios); if (ios/=0) stop 'convertgca:: error allocating labels'

    REWIND 4
    p=0
    DO
        READ (4,'(A)', IOSTAT=ios) aline
        IF (ios/=0) EXIT
        IF (aline(1:7) == "RESIDUE") THEN
            READ(aline(10:),*) p
            READ(aline(10:),*) labels(p)%gca_label, labels(p)%pdb_label, resid,phi,psi,omg,x,y,z
            if (x==0..and.y==0..and.z==0.) then
              labels(p)%pdb_label = -999
            endif
            IF (labels(p)%pdb_label==FIRSTres)start=p
        END IF
    END DO

    write(0,*) "start=",start," FIRSTres=",FIRSTres," gcaMAX=",gcaMAX

    REWIND 4
    p=0
    DO
      READ (4,'(A)', IOSTAT=ios) aline
      if (ios/=0) exit
      WRITE (7,'(A)')trim(aline)
      IF (aline(1:4) == "HMM:")  THEN  !! Place remarks after first line.
        WRITE (7,'(A,f8.3)') "REMARK  SC_CONTACT (min(sidechain,sidecahin))cutoff distance= ",cutoff
        WRITE (7,'(A,f8.3)') "REMARK  CA_CONTACT (C-alpha,C-alpha) cutoff distance= ",mindis
      ENDIF
      IF (aline(1:8) == "RESIDUE:" )  THEN
        READ(aline(10:),*) p
      ENDIF
      IF (aline(1:6) == "GAMMA:" )  THEN
        IF (labels(p)%pdb_label>=FIRSTres.and.labels(p)%pdb_label<=MAXres) THEN            
            numcontacts=count(SCcontact_map(labels(p)%pdb_label,FIRSTres:MAXres)==1)
            WRITE (7, '(A10, 1X, I4, 1X, I4, $)') "SC_CONTACT", labels(p)%gca_label, numcontacts
            DO i=start, gcaMAX
                IF ((labels(i)%pdb_label>=FIRSTres.and.labels(i)%pdb_label<=MAXres)) THEN
                  IF (SCcontact_map(labels(p)%pdb_label, labels(i)%pdb_label) == 1) &
                    WRITE (7, '(1X, I4, $)') labels(i)%gca_label
                END IF
            END DO
            WRITE (7,*) " "
            numcontacts=count(CAcontact_map(labels(p)%pdb_label,FIRSTres:MAXres)==1)
            WRITE (7, '(A10, 1X, I4, 1X, I4, $)') "CA_CONTACT", labels(p)%gca_label, numcontacts
            j = 0
            DO i=start, gcaMAX
                IF ((labels(i)%pdb_label>=FIRSTres.and.labels(i)%pdb_label<=MAXres)) THEN
                  IF (CAcontact_map(labels(p)%pdb_label, labels(i)%pdb_label) == 1) then
                    WRITE (7, '(1X, I4, $)') labels(i)%gca_label
                    j = j + 1
                  endif
                END IF
            END DO
            WRITE (7,*) " "
            if (j/=numcontacts) then
              write(0,*) "convertgca.f90:: error  --- j/=numcontacts",j,numcontacts," first,last=",FIRSTres,MAXres
              !! diagnostic
              DO i=start, gcaMAX
                IF (CAcontact_map(labels(p)%pdb_label, labels(i)%pdb_label) == 1) then
                  write(*,*) "pdb_label=",labels(i)%pdb_label," ::::: gca_label=",labels(i)%gca_label
                endif
              enddo
              stop 'convertgca.f90:: error  --- j/=numcontacts'
            endif
        ELSE
            WRITE (7, '(A10, 1X, I4, 1X, I3)') "CA_CONTACT", labels(p)%gca_label, 0
            WRITE (7, '(A10, 1X, I4, 1X, I3)') "SC_CONTACT", labels(p)%gca_label, 0
        END IF
        firstinstance=.false.
      END IF
    END DO
    if (allocated(labels)) DEALLOCATE (labels)
    close(7)
END SUBROUTINE contacts_to_gcb    
