#!/bin/csh
#### NOTE: this script uses standard unix commands awk, sed, grep, cut, basename, etc in addition
#### to executables. Must be run using csh.
#### Script to run SCALI non-sequential alignment
#### Sun Sep 11 18:06:39 EDT 2005
#### TOOLS
#### Last update: Mon Mar 17 08:34:51 EDT 2014

## Settings
setenv CONVERT "/usr/bin/convert"

## Set this path to the installation directory
setenv SCALIPATH /bach1/home/bystrc/server/scali/src
which $SCALIPATH/xscali >& /dev/null
if ($status) then
  echo  "The SCALI programs are not in the current directory!"
  echo -n "Where is the SCALI installation directory: "
  setenv SCALIPATH $<
  which $SCALIPATH/xscali >& /dev/null
  if ($status) then
    echo "Please do the installation."
    exit 
  else
    echo "OK. Proceeding."
  endif
endif

## ----- allgapless.f90 settings: -----
##Usage: xallgapless gca1 code1 gca2 code2 [lmin smin mdacut gscale bias]
## gca1 and gca2 are GCA-format sequence-structure files
## LMIN is the minimum allowable length of a gapless alignment. lmin= 5
## SMIN is the lowest allowable score for an alignment. smin= 0.0000000
## GSCALE= gamma-scale = relative weight of HMMSTR score versus Blosum score. gscale= 1.000000
## BIAS= matrix bias = a constant added to each match position.
setenv LMIN 5
setenv SMIN 0.001
setenv SCUT 0.00
setenv MDACUT 120.
setenv BIAS 0.0
setenv GWEIGHT 1.00

## SCALI settings
## If SCALI isn't performing well, try increasing NPICK
setenv SEQUENTIAL 1
setenv NPICK  50
setenv WEIGHT 0.5
setenv MSCORE 0.0
setenv NSPEN 10.0
setenv USESC 1  

if ($#argv < 4) then
  echo  "Batch usage:: runscali.csh  pdbfile1 ChainID1 pdbfile2 ChainID2 sequential=Y/N"
  echo  "========== Running interactively ========= "
  echo -n "Input two PDB files: template PDB file= "
  setenv PDB1 $<
  if !(-e $PDB1) then
    echo "File Not Found: $PDB1"
    echo "Please start over."
    exit
  endif
  echo -n "Which chainID for $PDB1 ? ["
  grep ^ATOM $PDB1 | cut -c 22-22 | uniq | awk '{printf "%s",$1}'
  echo -n "]"
  setenv CHAIN1 $<
  echo -n "target PDB file= "
  setenv PDB2 $<
  if !(-e $PDB2) then
    echo "File Not Found: $PDB2"
    echo "Please start over."
    exit
  endif
  echo -n "Which chainID for $PDB1 ? ["
  grep ^ATOM $PDB2 | cut -c 22-22 | uniq | awk '{printf "%s",$1}'
  echo -n "]"
  setenv CHAIN2 $<
else
  setenv PDB1 $1
  setenv CHAIN1 "$2"
  setenv PDB2 $3
  setenv CHAIN2 "$4"
endif

if ($#argv > 4) then
  setenv SEQUENTIAL $5
  if ($SEQUENTIAL == "n") setenv SEQUENTIAL 0
  if ($SEQUENTIAL == "y") setenv SEQUENTIAL 1
  if ($SEQUENTIAL == "N") setenv SEQUENTIAL 0
  if ($SEQUENTIAL == "Y") setenv SEQUENTIAL 1
  if ($SEQUENTIAL == "f") setenv SEQUENTIAL 0
  if ($SEQUENTIAL == "t") setenv SEQUENTIAL 1
  if ($SEQUENTIAL == "F") setenv SEQUENTIAL 0
  if ($SEQUENTIAL == "T") setenv SEQUENTIAL 1
endif

if ($#argv > 5) then
  setenv HTMLOUT $6
else
  setenv HTMLOUT "0"
endif

if ($SEQUENTIAL) then
  echo "========== SEQUENTIAL SCALI alignment ==========="
else
  echo " =========== NON-SEQUENTIAL SCALI alignment ==========="
endif

setenv NAME1 `basename $PDB1 .pdb`
setenv NAME2 `basename $PDB2 .pdb`

## generate sequences from PDB files
$SCALIPATH/x3to1 "$CHAIN1" < $PDB1 > $NAME1.seq
if (`grep -v ^">" $NAME1.seq  | wc -c ` < 1) then
  echo "ERROR::::::: Chain $CHAIN1 not found in PDB file $PDB1"
  setenv CHAIN1 "`head -1 $NAME1.seq | cut -c 21-21`"
  echo "Trying first chain instead: $CHAIN1"
  $SCALIPATH/x3to1 "$CHAIN1" < $PDB1 > $NAME1.seq
  if (`grep -v ^">" $NAME1.seq  | wc -c ` < 1) then
    echo "ERROR::::::: Chain $CHAIN1 not found in PDB file $PDB1"
    exit
  endif
endif
if ( "$CHAIN1" == "?" ) setenv CHAIN1 "`head -1 $NAME1.seq | cut -c 21-21`"
$SCALIPATH/x3to1 "$CHAIN2" < $PDB2 > $NAME2.seq
if (`grep -v ^">" $NAME2.seq  | wc -c ` < 1) then
  echo "ERROR::::::: Chain $CHAIN2 not found in PDB file $PDB2"
  setenv CHAIN2 "`head -1 $NAME2.seq | cut -c 21-21`"
  echo "Trying first chain instead: $CHAIN2"
  $SCALIPATH/x3to1 "$CHAIN2" < $PDB2 > $NAME2.seq
  if (`grep -v ^">" $NAME2.seq  | wc -c ` < 1) then
    echo "ERROR::::::: Chain $CHAIN2 not found in PDB file $PDB2"
    exit
  endif
endif
if ( "$CHAIN2" == "?" ) setenv CHAIN2 "`head -1 $NAME2.seq | cut -c 21-21`"

###================== Add PSI-Blast here. =====================
####### NOTE: Scali makes better alignments using profiles generated by Psi-Blast
####### Install Psi-Blast and run as follows to generate a profile. 
####### Then comment out the two 'xseq2drct' lines
####### blastpgp -i $NAME1.seq -o NAME1.blast -Q $NAME1.profile -j 2 -m 6 -I -b 1000 -e 0.1
####### $SCALIPATH/profile2drct.pl $NAME1.blast $NAME1.profile  $NAME1.drct $NAME1
####### blastpgp -i $NAME2.seq -o NAME2.blast -Q $NAME2.profile -j 2 -m 6 -I -b 1000 -e 0.1
####### $SCALIPATH/profile2drct.pl $NAME2.blast $NAME2.profile  $NAME2.drct $NAME2

## Make direct access binary sequence files
#$SCALIPATH/xseq2drct $NAME1.drct < $NAME1.seq
#$SCALIPATH/xseq2drct $NAME2.drct < $NAME2.seq

## Add coordinates to binary files
#$SCALIPATH/xgiali $NAME1.drct $PDB1 "$CHAIN1" $NAME1
#$SCALIPATH/xvallview $NAME1.drct | grep ">>>" | head
#$SCALIPATH/xgiali $NAME2.drct $PDB2 "$CHAIN2" $NAME2
#$SCALIPATH/xvallview $NAME2.drct | grep ">>>" | head

## Predict local structure using HMMSTR (Bystroff et al, J Mol Biol, 2000)
# Usage: xhmmstrgca model_R.hmm seqfile(fasta) gcafile profile(psi-blast) use_structure(HMMstr T/F) [pdbfile chain] 

$SCALIPATH/xhmmstrgca $SCALIPATH/hmmstr/model_R.hmm $NAME1.seq $NAME1.gca $PDB1 "$CHAIN1" T
$SCALIPATH/xhmmstrgca $SCALIPATH/hmmstr/model_R.hmm $NAME2.seq $NAME2.gca $PDB2 "$CHAIN2" T

## Generate contact maps for use by SCALI
## Usage: xconvertcij PDBfile chainID GCAin GCAout
($SCALIPATH/xgetchain "$CHAIN1" < $PDB1 > ${NAME1}_1.pdb ) |& grep "extracted from PDB" | awk '{print $1}' >& __linesfound__
if ( `cat __linesfound__ ` < 1 ) then
  echo "ERROR....... Chain $CHAIN1 not found in PDB file $PDB1"
  exit
endif
rm -f __linesfound__
$SCALIPATH/xconvertgca ${NAME1}_1.pdb "$CHAIN1" $NAME1.gca $NAME1.gcb
\mv $NAME1.gcb $NAME1.gca
($SCALIPATH/xgetchain "$CHAIN2" < $PDB2 > ${NAME2}_1.pdb ) |& grep "extracted from PDB" | awk '{print $1}' >& __linesfound__
if ( `cat __linesfound__ ` < 1 ) then
  echo "ERROR....... Chain $CHAIN2 not found in PDB file $PDB2"
  exit
endif
rm -f __linesfound__
$SCALIPATH/xconvertgca ${NAME2}_1.pdb "$CHAIN2" $NAME2.gca $NAME2.gcb
\mv $NAME2.gcb $NAME2.gca

## Generate set of all gapless alignments for SCALI
$SCALIPATH/xallgapless $NAME1.gca  $NAME1 $NAME2.gca $NAME2  $LMIN $SMIN $MDACUT $GWEIGHT $BIAS > ${NAME1}_${NAME2}.gapless
head -20 ${NAME1}_${NAME2}.gapless | grep ^"Length" > ${NAME1}_${NAME2}.sort
@ N = `head -1  ${NAME1}_${NAME2}.sort | tail -1 | awk '{print $(NF)}'`
@ M = `head -2  ${NAME1}_${NAME2}.sort | tail -1 | awk '{print $(NF)}'`
## Cut down (for speed) on the number of gapless alignments, Use 0.1 of L1 * L2
@ NALIGN = $N * $M / 10
echo "Keeping $NALIGN top gapless alignments"
grep ^"###" ${NAME1}_${NAME2}.gapless | awk -v mm=$MSCORE '$2 > mm' | sort -nrk 2 | head -$NALIGN >> ${NAME1}_${NAME2}.sort
setenv LOGFILE ${NAME1}_${NAME2}.log

## Run SCALI
echo "Running SCALI using Npick=$NPICK"
$SCALIPATH/xscali $NAME1 $NAME2 ${NAME1}_${NAME2}.sort $SEQUENTIAL $NSPEN $MSCORE  $WEIGHT  $NPICK $USESC > $LOGFILE
echo "Done... SCALI progress was sent to $LOGFILE"
 @ NBEST = `tail $LOGFILE | grep ^"BEST ALIGNMENT" | awk '{print $3}'`
sed -n "/BEST ALIGNMENT/,$ p" $LOGFILE > ${NAME1}_${NAME2}.best
sed -n "/Best_Alignment /,$ p" ${NAME1}${NAME2}.scali | grep -v "Alignment" > ${NAME1}_${NAME2}.ali

## Calculate RMSDs and make postscript images of the alignments.
$SCALIPATH/xovrlayali ${NAME1}_${NAME2}.ali ${NAME1}_1.pdb  "$CHAIN1" 1 ${NAME2}_1.pdb "$CHAIN2" 1 1000 ${NAME1}_${NAME2}.pdb
$SCALIPATH/xscali2ps ${NAME1}${NAME2}.scali ${NAME1}_${NAME2}.ps
$CONVERT -trim -geometry 300 ${NAME1}_${NAME2}.ps ${NAME1}_${NAME2}.jpg
$SCALIPATH/xscali2ps ${NAME1}_${NAME2}.ali ${NAME1}_${NAME2}_best.ps
$CONVERT -trim -geometry 300 ${NAME1}_${NAME2}_best.ps ${NAME1}_${NAME2}_best.jpg

## Report the filenames for SCALI alignments
echo "=============================================================================="
echo "SCALI is finished. Your superimposed coordinates are in ${NAME1}_${NAME2}.pdb "
echo "                   Your alignment in characters   is in ${NAME1}_${NAME2}.best "
echo "                   Your alignment in numbers      is in ${NAME1}_${NAME2}.ali "
echo "                   Color image of all alignments  is in ${NAME1}_${NAME2}.ps "
echo "                   Image of best alignment only   is in ${NAME1}_${NAME2}_best.ps "
echo "                   Best alignment in FASTA format is in ${NAME1}_${NAME2}.fasta "
echo "=============================================================================="
echo "SCALI alignment    "
cat ${NAME1}_${NAME2}.best

if !( "$HTMLOUT" == "0" ) then
  setenv OUTDIR `dirname $HTMLOUT`
  if (-e $OUTDIR/${NAME1}_${NAME2}_best.jpg ) rm -f $OUTDIR/${NAME1}_${NAME2}_best.jpg
  cp -f  ${NAME1}_${NAME2}_best.jpg $OUTDIR/
  if (-e $OUTDIR/${NAME1}_${NAME2}.jpg ) rm -f $OUTDIR/${NAME1}_${NAME2}.jpg
  cp -f  ${NAME1}_${NAME2}.jpg $OUTDIR/
  if (-e $OUTDIR/${NAME1}_${NAME2}.pdb      ) rm -f $OUTDIR/${NAME1}_${NAME2}.pdb
  cp -f  ${NAME1}_${NAME2}.pdb $OUTDIR/
  echo "########## web page output to $HTMLOUT ############"
  if (-e $HTMLOUT) rm -f $HTMLOUT
  echo '<\!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">         ' > $HTMLOUT
  echo ' ' >> $HTMLOUT
  echo ' ' >> $HTMLOUT
  echo '<html> ' >> $HTMLOUT
  echo '<head> ' >> $HTMLOUT
  echo '<title>SCALI alignment '${NAME1}" vs "${NAME2}'</title> ' >> $HTMLOUT
  echo '</head> ' >> $HTMLOUT
  echo '<body> ' >> $HTMLOUT
  echo '<A HREF="http://www.bioinfo.rpi.edu/bystrc/scali"><img src="banner.jpg" width=600 border=0></A><br> ' >> $HTMLOUT
  echo '<pre>' >> $HTMLOUT
  cat ${NAME1}_${NAME2}.best >> $HTMLOUT
  echo '</pre>' >> $HTMLOUT
  echo '<table>' >> $HTMLOUT
  echo '  <tr>' >> $HTMLOUT
  echo '    <td>' >> $HTMLOUT
  echo '       <h4>Best alignment ' >> $HTMLOUT
  setenv NBREAK `head -1 ${NAME1}_${NAME2}.best | awk '{print $7}'`
  if ( $NBREAK ) then
    echo '-- NON-SEQUENTIAL ' >> $HTMLOUT
    echo "$NBREAK breaks</h4>" >> $HTMLOUT
  else
    echo '-- Sequential </h4>' >> $HTMLOUT
  endif
  echo '    </td>' >> $HTMLOUT
  echo '    <td>' >> $HTMLOUT
  echo '       <h4>All alignments</h4>' >> $HTMLOUT
  echo '    </td>' >> $HTMLOUT
  echo '  </tr>' >> $HTMLOUT
  echo '  <tr>' >> $HTMLOUT
  echo '    <td>' >> $HTMLOUT
  echo '       <img src="'"${NAME1}_${NAME2}_best.jpg"'" border=1>' >> $HTMLOUT
  echo '    </td>' >> $HTMLOUT
  echo '    <td>' >> $HTMLOUT
  echo '       <img src="'"${NAME1}_${NAME2}.jpg"'" border=1>' >> $HTMLOUT
  echo '    </td>' >> $HTMLOUT
  echo '  </tr>' >> $HTMLOUT
  echo '  <tr>' >> $HTMLOUT
  echo '    <td>' >> $HTMLOUT
  echo '       <A href="'"${NAME1}_${NAME2}.pdb"'">superimposed coordinates</A>' >> $HTMLOUT
  echo '    </td>' >> $HTMLOUT
  echo '  </tr>' >> $HTMLOUT
  echo '</table>' >> $HTMLOUT
  echo '</body> ' >> $HTMLOUT
  echo '</html> ' >> $HTMLOUT
endif
### END OF SCRIPT  ###
