!!!! SCALI2PS
! Read many alignments.Plot alignment probabilities as colored boxes.
! C.Bystroff Fri Nov 25 08:36:07 EST 2005
!=========================================================

program scali2ps
!! From cij2ps.f90  CB. Sun Nov 13 19:21:49 EST 2005
implicit none
integer :: I,J,t,q,N1,N2,NQ,ounit=12,L
real,dimension(:,:),allocatable :: pij
character(len=80) :: scalifile,seqfile,psfile,str
character(len=800) :: seq1,seq2
real :: x
integer :: jarg,iargc,ios=0,N,ia,imax,jmax,nalign,i1,j1,nmatch
integer :: A(1000),B(1000)
character(len=200) :: aline
integer :: getfields

seq1=' '
seq2=' '

jarg=iargc()
if (jarg < 2) then
  write(*,'("Usage: xscali2ps scalifile psfile ")')
  stop 
endif

call getarg(1,scalifile)
call getarg(2,psfile)

!! Read alignment file
open(22,file=scalifile,status='old',form='formatted',iostat=ios)
if (ios/=0) stop 'scali2ps.f90:: Error opening scalifile'
ia = 0
do
    read(22,'(a)',iostat=ios) aline
    if (ios/=0) exit
    if (aline(1:9)=="Alignment") then
      ia = ia + 1
      cycle
    endif
    if (aline(1:9)=="Best_Alig") exit
    read(aline,*,iostat=ios) i1,j1
    if (ios/=0) stop 'scali2ps:  error parsing match'
    imax = max(i1,imax)
    jmax = max(j1,jmax)
    i = i + 1
!    pij(i1,j1) =  pij(i1,j1) + 1
enddo
rewind(22)
nalign = ia
if (nalign==0) nalign=1
if (imax==0 .or. jmax==0) stop 'scali2ps:: error. no aligned positions were input.'
allocate(pij(imax,jmax),stat=ios); if (ios/=0) stop 'scali2ps:: error allocating Pij'
ia = 0
pij = 0.
do
    read(22,'(a)',iostat=ios) aline
    if (ios/=0) exit
    if (aline(1:9)=="Alignment") then
      ia = ia + 1
      cycle
    endif
    if (aline(1:9)=="Best_Alig") exit
    read(aline,*,iostat=ios) i1,j1
    if (ios/=0) stop 'scali2ps:  error parsing match'
    if (i1==0.or.j1==0) cycle
    i = i + 1
    pij(i1,j1) =  pij(i1,j1) + 1
enddo
nmatch = nint(sum(pij))
write(*,'(i9," matches read from ",i3," alignments in file ",a)') nmatch, nalign, trim(scalifile)
close(22)
pij = pij/real(nalign)

call WRITEPS(psfile,pij,imax,jmax)

CONTAINS
!---------------------------------------------------------------!
subroutine WRITEPS(filename,aij,N,M)
! Zeros set to white. Others to hue scale from yellow to blue
implicit none
character(len=80),intent(in) :: filename
integer,intent(in) :: N,M
real,dimension(N,M),intent(in) :: aij
integer :: I,J,K
real,parameter :: cm=30 ! points/cm
real,parameter :: page=550 ! points/page
real,parameter :: margin=1. ! cm margin
real :: boxsize,x,y,g

open(22,file=filename,status="replace",form="formatted",iostat=ios)
if (ios/=0) stop "scali2ps:: Error opening psfile"
boxsize = ((page/max(N,M))/cm)
write(22,'("%!PS")')
write(22,'("% ---- define box procedure")')
write(22,'("/cm { ",i4," mul } def")') int(cm)
write(22,'("/box")')
write(22,'("{ ",f8.3," cm 0 rlineto")') boxsize
write(22,'("  0 ",f8.3," cm rlineto")') boxsize
write(22,'("  ",f8.3," cm 0 rlineto")') -boxsize
write(22,'("  closepath } def")')
write(22,'("/graybox")')
write(22,'("{ newpath setgray moveto box fill } def")')
write(22,'("/hsbbox")')
write(22,'("{ newpath sethsbcolor moveto box fill } def")')
write(22,'("/Times_Roman findfont")')
write(22,'(f8.3," cm scalefont setfont")') boxsize
write(22,'("% -- begin program ---- %")')
!  draw sequence 1 horizontal
y = margin - boxsize
if (y < 0) y = 0
do I=1,N
  x = margin + (I-1)*boxsize
  write(22,'(f8.2," cm ",f8.2," cm  moveto")') x,y
  write(22,'("(",a1,") show")') seq1(I:I)
enddo
!  draw sequence 2 vertical
x = margin - boxsize
if (x < 0) x = 0
do J=1,M
  y = margin + (J-1)*boxsize
  write(22,'(f8.2," cm ",f8.2," cm  moveto")') x,y
  write(22,'("(",a1,") show")') seq2(J:J)
enddo
do J=1,M
  ! y in units of cm
  y = margin + (J-1)*boxsize
  do I=1,N
    if (aij(I,J)<=0.) cycle
    x = margin +(I-1)*boxsize
    g = 0.20 - aij(I,J)*0.5   !! hue = yellow as aij approaches 0
    g = mod(g+1.00,1.00)
    write(22,'(f8.2," cm ",f8.2," cm ",3f6.3," hsbbox")') x,y,g,1.,1.
  enddo
enddo
write(22,'("showpage")')
close(22)
end subroutine WRITEPS
!---------------------------------------------------------------!
subroutine getseq(filename,seq,N)
character(len=80),intent(in) :: filename
integer,intent(in) :: N
character(len=N),intent(out) :: seq
integer :: I,J,ios=0,NL=30
integer :: A(100),B(100)
character(len=2850) :: aline
integer :: getfields

open(22,file=filename,status="old",form="formatted",iostat=ios)
if (ios/=0) stop "Error opening file 3"

do I=1,N
  aline = " "
  do while (aline(1:7)/="RESIDUE")
    read(22,'(a)',iostat=ios) aline
    if (ios/=0) then
      write(*,'("EOF on I=",i4," of ",a)') I,filename
      write(*,'("last line read",/,a)') aline
      stop "Error reading RESIDUE"
    endif
  enddo
  J = getfields(NL,aline,A,B)
  seq(I:I) = aline(A(4):B(4))
enddo
close(22)

end subroutine getseq
!---------------------------------------------------------------!
subroutine getgamma(filename,gamma,NQ,N)
integer,parameter :: MAXQ=282
integer,intent(in) :: N,NQ
character(len=80),intent(in) :: filename
real,dimension(NQ,N),intent(out) :: gamma
character(len=2850) :: aline
integer :: NL=MAXQ+2,A(MAXQ),B(MAXQ)
integer :: I,J,ios=0
integer :: getfields

open(22,file=filename,status="old",form="formatted",iostat=ios)
if (ios/=0) stop "Error opening file 2"

gamma = 0.0
do I=1,N
  aline = " "
  do while (aline(1:5)/="GAMMA")
    read(22,'(a)',iostat=ios) aline
    if (ios/=0) then
      write(*,'("EOF on I=",i4," of ",a)') I,filename
      write(*,'("last line read",/,a)') aline
      stop "Error reading gamma"
    endif
  enddo
  J = getfields(NL,aline,A,B)
  read(aline(A(2):),*,iostat=ios) gamma(1:NQ,I)
  if (ios/=0) then
    write(*,'("EOL on gamma line ",i3)') I
    write(*,'("gamma line ",a)') aline
    write(*,'("gamma = ",9999f7.4)') gamma(1:NQ,I)
    stop "Error reading gamma line"
  endif
  ! diagnostic 
  write(*,'(f9.5,$)') gamma(NQ,I)
  !if (J-2 < NQ) then
  !  write(*,'("EOL on J-2 =",i4," of gamma line ",i3)') J-2,I
  !  stop "Error reading gamma line"
  !endif
  !do J=1,NQ
  !  read(aline(A(J+2):B(J+2),*,iostat=ios) gamma(
enddo
close(22)
  
  

end subroutine getgamma
!---------------------------------------------------------------!
subroutine getsmat(mfile,smat,NQ)
character(len=80),intent(in) :: mfile
integer,intent(in) :: NQ
integer :: I,J,ios
real,dimension(NQ,NQ),intent(inout) :: smat

open(unit=99,file=mfile,status='old',form='formatted',iostat=ios)
if (ios/=0) stop "Error opening file mfile"

do I=2,NQ  
   read(99,*,iostat=ios) smat(2:NQ,I)
   if (ios/=0) stop 'Error reading a line from mfile'
enddo
write(*,'("Done reading ",a)') mfile

close(99)


end subroutine getsmat
!------------------------------------------------------------!

subroutine getnstates(filename,NQ,N)
implicit none
character(len=80),intent(in) :: filename
integer,intent(out) :: NQ,N
integer :: NL=10
integer :: ios=0,A(20),B(20)
character(len=2000) :: aline
integer :: getfields

open(22,file=filename,status="old",form="formatted",iostat=ios)
if (ios/=0) stop "error opening file 1"

aline = " "
do while (aline(1:3)/="HMM")
  read(22,'(a)',iostat=ios) aline
  if (ios/=0) stop "HMM line not found"
enddo
NL = getfields(NL,aline,A,B)
read(aline(A(4):B(4)),*,iostat=ios) NQ

rewind(22)
aline = " "
do while (aline(1:3)/="GCA")
  read(22,'(a)',iostat=ios) aline
  if (ios/=0) stop "GCA line not found"
enddo
NL = 10
NL = getfields(NL,aline,A,B)
read(aline(A(5):B(5)),*,iostat=ios) N

close(22)
 
end subroutine getnstates
end program scali2ps


!************************************************************************
!* find the limits of up to N fields of non-blank
!* characters in the string 'aline' . The start of
!* field I is A(I), the end is B(I)
        integer function getfields(N,aline,A,B)
        implicit none
        integer,intent(inout) ::  N,A(N),B(N)
        integer ::  I,FF,LL
        character(len=*),intent(in) :: aline
!**
        I = 1
        FF = 0
        LL = len(aline)
        do while (FF.lt.N.and.I.le.LL)
          do while (aline(I:I).eq.' '.and.I.le.LL)
            I = I + 1
          enddo
          if (I.le.LL) then
            FF = FF+1
            A(FF) = I
            do while (aline(I:I).ne.' '.and.I.le.LL)
              I = I + 1
            enddo
            B(FF) = I-1
          endif
        enddo
        getfields = FF
        end function getfields
!************************************************************************
